@@ -41,6 +41,7 @@ public:
 		update_metadata_ = false;
 		is_user_request_ = true;
 		is_redirect_ = false;
+		need_retry_ = false;
 	}
 
 protected:
@@ -103,10 +104,19 @@ private:
 	virtual int first_timeout();
 	bool has_next();
 	bool check_redirect();
+	bool process_produce();
+	bool process_fetch();
+	bool process_metadata();
+	bool process_find_coordinator();
+	bool process_join_group();
+	bool process_sync_group();
+	bool process_sasl_authenticate();
+	bool process_sasl_handshake();
 
 	bool update_metadata_;
 	bool is_user_request_;
 	bool is_redirect_;
+	bool need_retry_;
 };
 
 CommMessageOut *__ComplexKafkaTask::message_out()
@@ -194,21 +204,45 @@ CommMessageOut *__ComplexKafkaTask::message_out()
 
 		while ((toppar = req->get_toppar_list()->get_next()) != NULL)
 		{
-			if (toppar->get_offset() == -1)
+			if (toppar->get_low_watermark() < 0)
+				toppar->set_offset_timestamp(KAFKA_TIMESTAMP_EARLIEST);
+			else if (toppar->get_high_watermark() < 0)
+				toppar->set_offset_timestamp(KAFKA_TIMESTAMP_LATEST);
+			else if (toppar->get_offset() == KAFKA_OFFSET_UNINIT)
 			{
-				if (toppar->get_offset_timestamp() == KAFKA_TIMESTAMP_UNINIT)
+				long long conf_ts =
+					this->get_req()->get_config()->get_offset_timestamp();
+				if (conf_ts == KAFKA_TIMESTAMP_EARLIEST)
 				{
-					long long conf_ts = this->get_req()->get_config()->get_offset_timestamp();
-					if (conf_ts != KAFKA_TIMESTAMP_UNINIT)
-						toppar->set_offset_timestamp(conf_ts);
-					else
-						toppar->set_offset_timestamp(KAFKA_TIMESTAMP_EARLIEST);
+					toppar->set_offset(toppar->get_low_watermark());
+					continue;
+				}
+				else if (conf_ts == KAFKA_TIMESTAMP_LATEST)
+				{
+					toppar->set_offset(toppar->get_high_watermark());
+					continue;
 				}
 				else
-					toppar->set_offset_timestamp(KAFKA_TIMESTAMP_LATEST);
+				{
+					toppar->set_offset_timestamp(conf_ts);
+				}
+			}
+			else if (toppar->get_offset() == KAFKA_OFFSET_OVERFLOW)
+			{
+				if (this->get_req()->get_config()->get_offset_timestamp() ==
+					KAFKA_TIMESTAMP_EARLIEST)
+				{
+					toppar->set_offset(toppar->get_low_watermark());
+				}
+				else
+				{
+					toppar->set_offset(toppar->get_high_watermark());
+				}
 			}
 			else
+			{
 				continue;
+			}
 
 			toppar_list.add_item(*toppar);
 			flag = true;
@@ -366,167 +400,210 @@ bool __ComplexKafkaTask::check_redirect()
 	}
 }
 
+bool __ComplexKafkaTask::process_find_coordinator()
+{
+	if (this->get_resp()->get_cgroup()->get_error())
+	{
+		this->error = this->get_resp()->get_cgroup()->get_error();
+		this->state = WFT_STATE_TASK_ERROR;
+		return false;
+	}
+	else
+	{
+		is_redirect_ = check_redirect();
+		this->get_req()->set_api_type(Kafka_JoinGroup);
+		return true;
+	}
+}
+
+bool __ComplexKafkaTask::process_join_group()
+{
+	KafkaResponse *msg = this->get_resp();
+	if (!msg->get_cgroup()->get_coordinator()->is_to_addr())
+	{
+		struct sockaddr_storage addr;
+		socklen_t addrlen = sizeof addr;
+		const struct sockaddr *paddr = (const struct sockaddr *)&addr;
+		this->get_peer_addr((struct sockaddr *)&addr, &addrlen);
+		msg->get_cgroup()->get_coordinator()->set_broker_addr(paddr, addrlen);
+		msg->get_cgroup()->get_coordinator()->set_to_addr(1);
+	}
+
+	if (msg->get_cgroup()->get_error() == KAFKA_MISSING_TOPIC)
+	{
+		this->get_req()->set_api_type(Kafka_Metadata);
+		this->get_req()->set_alien();
+		update_metadata_ = true;
+	}
+	else if (msg->get_cgroup()->get_error() == KAFKA_MEMBER_ID_REQUIRED)
+	{
+		this->get_req()->set_api_type(Kafka_JoinGroup);
+	}
+	else if (msg->get_cgroup()->get_error() == KAFKA_UNKNOWN_MEMBER_ID)
+	{
+		msg->get_cgroup()->set_member_id("");
+		this->get_req()->set_api_type(Kafka_JoinGroup);
+	}
+	else if (msg->get_cgroup()->get_error())
+	{
+		this->error = msg->get_cgroup()->get_error();
+		this->state = WFT_STATE_TASK_ERROR;
+		return false;
+	}
+	else
+	{
+		this->get_req()->set_api_type(Kafka_SyncGroup);
+	}
+	return true;
+}
+
+bool __ComplexKafkaTask::process_sync_group()
+{
+	if (this->get_resp()->get_cgroup()->get_error())
+	{
+		this->error = this->get_resp()->get_cgroup()->get_error();
+		this->state = WFT_STATE_TASK_ERROR;
+		return false;
+	}
+	else
+	{	
+		this->get_req()->set_api_type(Kafka_OffsetFetch);
+		return true;
+	}
+}
+
+bool __ComplexKafkaTask::process_metadata()
+{
+	KafkaResponse *msg = this->get_resp();
+	if (update_metadata_)
+	{
+		KafkaCgroup *cgroup = msg->get_cgroup();
+		if (cgroup->run_assignor(msg->get_meta_list(),
+								 msg->get_alien_meta_list(),
+								 cgroup->get_protocol_name()) < 0)
+		{
+			this->error = errno;
+			this->state = WFT_STATE_TASK_ERROR;
+		}
+		else
+		{
+			this->get_req()->set_api_type(Kafka_SyncGroup);
+		}
+		return true;
+	}
+	else
+	{
+		msg->get_meta_list()->rewind();
+		KafkaMeta *meta;
+		while ((meta = msg->get_meta_list()->get_next()) != NULL)
+		{
+			if (meta->get_error() == KAFKA_LEADER_NOT_AVAILABLE)
+			{
+				this->get_req()->set_api_type(Kafka_Metadata);
+				return true;
+			}
+		}
+		return false;
+	}
+}
+
+bool __ComplexKafkaTask::process_fetch()
+{
+	bool ret = false;
+	KafkaToppar *toppar;
+	this->get_resp()->get_toppar_list()->rewind();
+	while ((toppar = this->get_resp()->get_toppar_list()->get_next()) != NULL)
+	{
+		if (toppar->get_error() == KAFKA_OFFSET_OUT_OF_RANGE)
+		{
+			toppar->set_offset(KAFKA_OFFSET_OVERFLOW);
+			toppar->set_low_watermark(KAFKA_OFFSET_UNINIT);
+			need_retry_ = true;
+			ret = true;
+		}
+	}
+	return ret;
+}
+
+bool __ComplexKafkaTask::process_produce()
+{
+	KafkaToppar *toppar;
+	this->get_resp()->get_toppar_list()->rewind();
+	while ((toppar = this->get_resp()->get_toppar_list()->get_next()) != NULL)
+	{
+		if (!toppar->record_reach_end())
+		{
+			this->get_req()->set_api_type(Kafka_Produce);
+			return true;
+		}
+	}
+	return false;
+}
+
+bool __ComplexKafkaTask::process_sasl_handshake()
+{
+	if (this->get_resp()->get_broker()->get_error())
+	{
+		this->error = this->get_resp()->get_broker()->get_error();
+		this->state = WFT_STATE_TASK_ERROR;
+		return false;
+	}
+	return true;
+}
+
+bool __ComplexKafkaTask::process_sasl_authenticate()
+{
+	if (this->get_resp()->get_broker()->get_error())
+	{
+		this->error = this->get_resp()->get_broker()->get_error();
+		this->state = WFT_STATE_TASK_ERROR;
+	}
+	return false;
+}
 bool __ComplexKafkaTask::has_next()
 {
-	bool ret = true;
-	KafkaResponse *msg = this->get_resp();
-
 	struct sockaddr_storage addr;
 	socklen_t addrlen = sizeof addr;
-	const struct sockaddr *paddr = (const struct sockaddr *)&addr;
-
 	//always success
 	this->get_peer_addr((struct sockaddr *)&addr, &addrlen);
 
-	if (!msg->get_broker()->is_to_addr())
+	const struct sockaddr *paddr = (const struct sockaddr *)&addr;
+	if (!this->get_resp()->get_broker()->is_to_addr())
 	{
-		msg->get_broker()->set_broker_addr(paddr, addrlen);
-		msg->get_broker()->set_to_addr(1);
+		this->get_resp()->get_broker()->set_broker_addr(paddr, addrlen);
+		this->get_resp()->get_broker()->set_to_addr(1);
 	}
 
-	switch (msg->get_api_type())
+	switch (this->get_resp()->get_api_type())
 	{
-	case Kafka_FindCoordinator:
-		if (msg->get_cgroup()->get_error())
-		{
-			this->error = msg->get_cgroup()->get_error();
-			this->state = WFT_STATE_TASK_ERROR;
-			ret = false;
-		}
-		else
-		{
-			is_redirect_ = check_redirect();
-			this->get_req()->set_api_type(Kafka_JoinGroup);
-		}
-
-		break;
-
-	case Kafka_JoinGroup:
-		if (!msg->get_cgroup()->get_coordinator()->is_to_addr())
-		{
-			msg->get_cgroup()->get_coordinator()->set_broker_addr(paddr, addrlen);
-			msg->get_cgroup()->get_coordinator()->set_to_addr(1);
-		}
-
-		if (msg->get_cgroup()->get_error() == KAFKA_MISSING_TOPIC)
-		{
-			this->get_req()->set_api_type(Kafka_Metadata);
-			update_metadata_ = true;
-		}
-		else if (msg->get_cgroup()->get_error() == KAFKA_MEMBER_ID_REQUIRED)
-		{
-			this->get_req()->set_api_type(Kafka_JoinGroup);
-		}
-		else if (msg->get_cgroup()->get_error() == KAFKA_UNKNOWN_MEMBER_ID)
-		{
-			msg->get_cgroup()->set_member_id("");
-			this->get_req()->set_api_type(Kafka_JoinGroup);
-		}
-		else if (msg->get_cgroup()->get_error())
-		{
-			this->error = msg->get_cgroup()->get_error();
-			this->state = WFT_STATE_TASK_ERROR;
-			ret = false;
-		}
-		else
-			this->get_req()->set_api_type(Kafka_SyncGroup);
-
-		break;
-
-	case Kafka_SyncGroup:
-		if (msg->get_cgroup()->get_error())
-		{
-			this->error = msg->get_cgroup()->get_error();
-			this->state = WFT_STATE_TASK_ERROR;
-			ret = false;
-		}
-		else
-			this->get_req()->set_api_type(Kafka_OffsetFetch);
-
-		break;
-
-	case Kafka_Metadata:
-		if (update_metadata_)
-		{
-			KafkaCgroup *cgroup = msg->get_cgroup();
-			if (cgroup->run_assignor(msg->get_meta_list(),
-									 cgroup->get_protocol_name()) < 0)
-			{
-				this->error = errno;
-				this->state = WFT_STATE_TASK_ERROR;
-			}
-			else
-				this->get_req()->set_api_type(Kafka_SyncGroup);
-		}
-		else
-		{
-			ret = false;
-			msg->get_meta_list()->rewind();
-			KafkaMeta *meta;
-
-			while ((meta = msg->get_meta_list()->get_next()) != NULL)
-			{
-				if (meta->get_error() == KAFKA_LEADER_NOT_AVAILABLE)
-				{
-					ret = true;
-					this->get_req()->set_api_type(Kafka_Metadata);
-					break;
-				}
-			}
-		}
-		break;
-
-	case Kafka_SaslHandshake:
-		if (msg->get_broker()->get_error())
-		{
-			this->error = msg->get_broker()->get_error();
-			this->state = WFT_STATE_TASK_ERROR;
-			ret = false;
-		}
-
-		break;
-
 	case Kafka_Produce:
-		{
-			msg->get_toppar_list()->rewind();
-			KafkaToppar *toppar;
-			while ((toppar = msg->get_toppar_list()->get_next()) != NULL)
-			{
-				if (!toppar->record_reach_end())
-				{
-					this->get_req()->set_api_type(Kafka_Produce);
-					return true;
-				}
-			}
-		}
-
-	case Kafka_SaslAuthenticate:
-		if (msg->get_broker()->get_error())
-		{
-			this->error = msg->get_broker()->get_error();
-			this->state = WFT_STATE_TASK_ERROR;
-		}
-
-		ret = false;
-		break;
-
+		return this->process_produce();
 	case Kafka_Fetch:
+		return this->process_fetch();
+	case Kafka_Metadata:
+		return this->process_metadata();
+	case Kafka_FindCoordinator:
+		return this->process_find_coordinator();
+	case Kafka_JoinGroup:
+		return this->process_join_group();
+	case Kafka_SyncGroup:
+		return this->process_sync_group();
+	case Kafka_SaslHandshake:
+		return this->process_sasl_handshake();
+	case Kafka_SaslAuthenticate:
+		return this->process_sasl_authenticate();
 	case Kafka_OffsetCommit:
 	case Kafka_OffsetFetch:
 	case Kafka_ListOffsets:
 	case Kafka_Heartbeat:
 	case Kafka_LeaveGroup:
 	case Kafka_ApiVersions:
-		ret = false;
-		break;
-
+		return false;
 	default:
-		ret = false;
 		this->state = WFT_STATE_TASK_ERROR;
 		this->error = WFT_ERR_KAFKA_API_UNKNOWN;
-		break;
+		return false;
 	}
-	return ret;
 }
 
 bool __ComplexKafkaTask::finish_once()
@@ -549,6 +626,12 @@ bool __ComplexKafkaTask::finish_once()
 				return true;
 			}
 
+			if (need_retry_)
+			{
+				is_user_request_ = false;
+				need_retry_ = false;
+			}
+
 			this->clear_resp();
 			return false;
 		}
